///////////////////////////////////////////////////////////////////////////////
//  Filename: $RCSfile: IOType.java,v $
//  Purpose:  Atom representation.
//  Language: Java
//  Compiler: JDK 1.4
//  Authors:  Joerg K. Wegner
//  Version:  $Revision: 1.10 $
//            $Date: 2004/07/25 20:43:19 $
//            $Author: wegner $
//  Original Author: ???, OpenEye Scientific Software
//  Original Version: babel 2.0a1
//
//  Copyright (c) Dept. Computer Architecture, University of Tuebingen, Germany
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
///////////////////////////////////////////////////////////////////////////////
package joelib.io;

import org.apache.log4j.Category;


/**
 * Input/output type for molecules.
 *
 * @author     wegnerj
 * @license GPL
 * @cvsversion    $Revision: 1.10 $, $Date: 2004/07/25 20:43:19 $
 */
public final class IOType implements java.io.Serializable
{
    //~ Static fields/initializers /////////////////////////////////////////////

    // Obtain a suitable logger.
    private static Category logger = Category.getInstance("joelib.io.IOType");

    //~ Instance fields ////////////////////////////////////////////////////////

    private String name;
    private String representation;
    private int typeNumber;

    //~ Constructors ///////////////////////////////////////////////////////////

    /**
     *  Constructor for the IOType object
     *
     * @param  _value           Description of the Parameter
     * @param  _name            Description of the Parameter
     * @param  _representation  Description of the Parameter
     */
    protected IOType(String _name, String _representation, int _typeNumber)
    {
        if (logger.isDebugEnabled())
        {
            logger.debug("Initializing " + this.getClass().getName());
        }

        typeNumber = _typeNumber;
        name = _name;
        representation = _representation;
    }

    //~ Methods ////////////////////////////////////////////////////////////////

    /**
     *  Gets the name attribute of the IOType class
     *
     * @param  type  Description of the Parameter
     * @return       The name value
     */
    public String getName()
    {
        return name;
    }

    /**
     *  Sets the representation attribute of the IOType object
     *
     * @param  _representation  The new representation value
     */
    public void setRepresentation(String _representation)
    {
        representation = _representation;
    }

    /**
     *  Gets the representation attribute of the IOType class
     *
     * @param  type  Description of the Parameter
     * @return       The representation value
     */
    public String getRepresentation()
    {
        return representation;
    }

    /**
     *  Gets the type attribute of the IOType class
     *
     * @param  _value  Description of the Parameter
     * @return         The type value
     */
    public int getTypeNumber()
    {
        return typeNumber;
    }

    /**
     *  Description of the Method
     *
     * @param  is  Description of the Parameter
     * @return     Description of the Return Value
     */
    public boolean equals(IOType is)
    {
        if (is instanceof IOType && (is != null))
        {
            if (is.typeNumber == typeNumber)
            {
                return true;
            }
        }

        return false;
    }

    public String toString()
    {
        return name;
    }
}
///////////////////////////////////////////////////////////////////////////////
//  END OF FILE.
///////////////////////////////////////////////////////////////////////////////
