///////////////////////////////////////////////////////////////////////////////
//Filename: $RCSfile: CDOAcceptedObjects.java,v $
//Purpose:  Chemical Markup Language.
//Language: Java
//Compiler: JDK 1.4
//Authors:  steinbeck@ice.mpg.de, gezelter@maul.chem.nd.edu,
//			egonw@sci.kun.nl, wegnerj@informatik.uni-tuebingen.de
//Version:  $Revision: 1.4 $
//			$Date: 2003/08/19 13:11:26 $
//			$Author: wegner $
//
//Copyright (C) 1997-2003  The Chemistry Development Kit (CDK) project
//Copyright (c) Dept. Computer Architecture, University of Tuebingen, Germany
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public License
//as published by the Free Software Foundation; either version 2.1
//of the License, or (at your option) any later version.
//All we ask is that proper credit is given for our work, which includes
//- but is not limited to - adding the above copyright notice to the beginning
//of your source code files, and to any copyright notice that you may distribute
//with programs based on this work.
//  
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Lesser General Public License for more details.
//  
//You should have received a copy of the GNU Lesser General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
///////////////////////////////////////////////////////////////////////////////
package joelib.io.types.cml;

import java.util.Enumeration;
import java.util.Vector;


/**
 * List of names (String classes) of objects accepted by CDO.
 *
 * @author egonw
 * @author c.steinbeck@uni-koeln.de
 * @author gezelter@maul.chem.nd.edu
 * @author     wegnerj
 * @license LGPL
 * @cvsversion    $Revision: 1.4 $, $Date: 2003/08/19 13:11:26 $
 */
public class CDOAcceptedObjects
{
    //~ Instance fields ////////////////////////////////////////////////////////

    private Vector objects;

    //~ Constructors ///////////////////////////////////////////////////////////

    /**
     * Constructor.
     */
    public CDOAcceptedObjects()
    {
        objects = new Vector();
    }

    //~ Methods ////////////////////////////////////////////////////////////////

    /**
     * Adds the name of an accepted object.
     *
     * @param object Name of the object
     */
    public void add(String object)
    {
        objects.addElement(object);
    }

    /**
     * Determine if an object name is contained in this list.
     *
     * @param   object Name of the object to search in the list
     * @return         true if the object is in the list, false otherwise
     */
    public boolean contains(String object)
    {
        return objects.contains(object);
    }

    /**
     * Returns the names in this list as a Enumeration class. Each element in the
     * Enumeration is of type String.
     *
     * @return The names of the accepted objects
     */
    public Enumeration elements()
    {
        return objects.elements();
    }
}
///////////////////////////////////////////////////////////////////////////////
//  END OF FILE.
///////////////////////////////////////////////////////////////////////////////
