///////////////////////////////////////////////////////////////////////////////
//  Filename: $RCSfile: Combine.java,v $
//  Purpose:  Counts the number of descriptors and molecules in a molecule file.
//  Language: Java
//  Compiler: JDK 1.4
//  Authors:  Joerg K. Wegner
//  Version:  $Revision: 1.9 $
//            $Date: 2004/07/25 20:43:27 $
//            $Author: wegner $
//
//  Copyright (c) Dept. Computer Architecture, University of Tuebingen, Germany
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
///////////////////////////////////////////////////////////////////////////////
package joelib.test;

import wsi.ra.tool.StopWatch;

/*==========================================================================*
 * IMPORTS
 *==========================================================================      */
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import org.apache.log4j.Category;

import joelib.data.JOEDataType;
import joelib.data.JOEGenericData;
import joelib.data.JOEPairData;

import joelib.io.IOType;
import joelib.io.IOTypeHolder;
import joelib.io.JOEFileFormat;
import joelib.io.MoleculeFileType;
import joelib.io.MoleculeIOException;

import joelib.molecule.JOEMol;

import joelib.util.iterator.GenericDataIterator;


/*==========================================================================*
 * CLASS DECLARATION
 *==========================================================================      */

/**
 * Example for converting molecules.
 *
 * @author     wegnerj
 * @license    GPL
 * @cvsversion    $Revision: 1.9 $, $Date: 2004/07/25 20:43:27 $
 */
public class Combine
{
    //~ Static fields/initializers /////////////////////////////////////////////

    /*-------------------------------------------------------------------------*
     * private static member variables
     *-------------------------------------------------------------------------      */

    // Obtain a suitable logger.
    private static Category logger = Category.getInstance("joelib.test.Combine");

    //~ Instance fields ////////////////////////////////////////////////////////

    /**
     *  Description of the Field
     */
    public PrintStream generatedOutput;
    private IOType inType1;
    private IOType inType2;
    private IOType outType;
    private String inputFile1;
    private String inputFile2;
    private String inputTypeName1;
    private String inputTypeName2;
    private String outputFile;
    private boolean checkIfSameTitle = true;

    //~ Constructors ///////////////////////////////////////////////////////////

    /**
     *Constructor for the ConvertSkip object
     */
    public Combine()
    {
    }

    //~ Methods ////////////////////////////////////////////////////////////////

    /*-------------------------------------------------------------------------*
     * main
     *-------------------------------------------------------------------------      */

    /**
     *  The main program for the ConvertSkip class
     *
     * @param args  The command line arguments
     */
    public static void main(String[] args)
    {
        Combine combine = new Combine();

        if (args.length != 6)
        {
            combine.usage();
            System.exit(0);
        }
        else
        {
            if (combine.parseCommandLine(args))
            {
                combine.test();
            }
            else
            {
                System.exit(1);
            }
        }
    }

    /**
     *  Description of the Method
     *
     * @param args  Description of the Parameter
     * @return      Description of the Return Value
     */
    public boolean parseCommandLine(String[] args)
    {
        if (args[0].indexOf("-i") == 0)
        {
            inputTypeName1 = args[0].substring(2);
            inType1 = IOTypeHolder.instance().getIOType(inputTypeName1);

            if (inType1 == null)
            {
                logger.error("Input type '" + inputTypeName1 +
                    "' not defined.");

                return false;
            }
        }

        inputFile1 = args[1];

        if (args[2].indexOf("-i") == 0)
        {
            inputTypeName2 = args[2].substring(2);
            inType2 = IOTypeHolder.instance().getIOType(inputTypeName2);

            if (inType2 == null)
            {
                logger.error("Input type '" + inputTypeName2 +
                    "' not defined.");

                return false;
            }
        }

        inputFile2 = args[3];

        if (args[4].indexOf("-o") == 0)
        {
            String outTypeS = args[4].substring(2);
            outType = IOTypeHolder.instance().getIOType(outTypeS.toUpperCase());

            if (outType == null)
            {
                logger.error("Output type '" + outTypeS + "' not defined.");

                return false;
            }
        }

        outputFile = args[5];

        return true;
    }

    public void test()
    {
        if ((inType1 == null) || (inputFile1 == null) || (inType2 == null) ||
                (inputFile2 == null) || (outType == null) ||
                (outputFile == null))
        {
            logger.error("Not correctly initialized.");
            logger.error("input type 1: " + inType1.getName());
            logger.error("input file 1: " + inputFile1);
            logger.error("input type 2: " + inType2.getName());
            logger.error("input file 2: " + inputFile2);
            logger.error("output type: " + outType.getName());
            logger.error("output file: " + outputFile);
            System.exit(1);
        }

        // get molecules from resource URL
        //    BufferedInputStream   in       = null;
        //    BufferedOutputStream  out      = null;
        FileInputStream in1 = null;
        FileInputStream in2 = null;
        FileOutputStream out = null;

        // get molecule loader/writer
        MoleculeFileType loader1 = null;
        MoleculeFileType loader2 = null;
        MoleculeFileType writer = null;

        try
        {
            //      in = new BufferedInputStream(new FileInputStream(inputFile));
            //      out = new BufferedOutputStream( new FileOutputStream(outputFile));
            in1 = new FileInputStream(inputFile1);
            in2 = new FileInputStream(inputFile2);
            out = new FileOutputStream(outputFile);
            loader1 = JOEFileFormat.getMolReader(in1, inType1);
            loader2 = JOEFileFormat.getMolReader(in2, inType2);
            writer = JOEFileFormat.getMolWriter(out, outType);
        }
         catch (Exception ex)
        {
            ex.printStackTrace();
        }

        if (!loader1.readable())
        {
            logger.error(inType1.getRepresentation() + " is not readable.");
            logger.error("You're invited to write one !;-)");
            System.exit(1);
        }

        if (!loader2.readable())
        {
            logger.error(inType2.getRepresentation() + " is not readable.");
            logger.error("You're invited to write one !;-)");
            System.exit(1);
        }

        if (!writer.writeable())
        {
            logger.error(outType.getRepresentation() + " is not writeable.");
            logger.error("You're invited to write one !;-)");
            System.exit(1);
        }

        // load molecules and handle test
        JOEMol mol1 = new JOEMol(inType1, outType);
        JOEMol mol2 = new JOEMol(inType2, outType);
        boolean success = true;
        StopWatch watch = new StopWatch();
        int molCounter = 0;
        logger.info("Start file combination ...");

        for (;;)
        {
            mol1.clear();
            mol2.clear();

            try
            {
                success = loader1.read(mol1);

                if (!success)
                {
                    break;
                }

                success = loader2.read(mol2);

                if (!success)
                {
                    break;
                }

                //                if (mol1.empty())
                //                {
                //                    logger.warn("Empty molecule.");
                //                    System.exit(1);
                //                }
                //                if (mol2.empty())
                //                {
                //                    logger.warn("Empty molecule.");
                //                    System.exit(1);
                //                }
                // check if the title of the molecule is equal
                if (checkIfSameTitle)
                {
                    if ((mol1.getTitle() != null) && (mol2.getTitle() != null))
                    {
                        if (!mol1.getTitle().equals(mol2.getTitle()))
                        {
                            logger.warn("Different molecules: " +
                                mol1.getTitle() + " " + mol2.getTitle() + " ?");

                            //continue;
                        }
                    }
                }

                //        if (VERBOSE)
                //        {
                //          System.out.println("write " + mol1.toString(verboseType));
                //        }
                // combine descriptor data
                GenericDataIterator gdit = mol2.genericDataIterator();
                JOEPairData pairData;
                JOEGenericData genericData;

                while (gdit.hasNext())
                {
                    genericData = gdit.nextGenericData();

                    if (genericData.getDataType() == JOEDataType.JOE_PAIR_DATA)
                    {
                        pairData = (JOEPairData) genericData;

                        JOEPairData dp = new JOEPairData();
                        dp.setAttribute(genericData.getAttribute());
                        dp.setValue(pairData.toString(outType));
                        mol1.addData(dp);
                    }
                }

                // write combineddata
                success = writer.write(mol1);

                if (!success)
                {
                    break;
                }

                molCounter++;
            }
             catch (IOException ex)
            {
                ex.printStackTrace();
                System.exit(1);
            }
             catch (MoleculeIOException ex)
            {
                ex.printStackTrace();
                molCounter++;
                logger.info("Molecule entry (#" + molCounter +
                    ") was skipped: " + mol1.getTitle());

                continue;
            }

            if ((molCounter % 500) == 0)
            {
                logger.info("... " + molCounter +
                    " molecules successful combined in " +
                    watch.getPassedTime() + " ms.");
            }
        }

        logger.info("... " + molCounter + " molecules successful combined in " +
            watch.getPassedTime() + " ms.");
    }

    /*-------------------------------------------------------------------------*
     * public methods
     *-------------------------------------------------------------------------      */

    /**
     *  Description of the Method
     */
    public void usage()
    {
        StringBuffer sb = new StringBuffer();
        String programName = this.getClass().getName();

        sb.append("Usage is :\n");
        sb.append("java -cp . ");
        sb.append(programName);
        sb.append(" -i<inputFormat>");
        sb.append(" <input file 1>");
        sb.append(" -i<inputFormat>");
        sb.append(" <input file 2>");
        sb.append(" -o<outputFormat>");
        sb.append(" <output file>");
        sb.append("\n\nSupported molecule types:");
        sb.append(IOTypeHolder.instance().toString());
        sb.append(
            "\n\nThis is version $Revision: 1.9 $ ($Date: 2004/07/25 20:43:27 $)\n");

        System.out.println(sb.toString());

        System.exit(0);
    }
}
///////////////////////////////////////////////////////////////////////////////
//  END OF FILE.
///////////////////////////////////////////////////////////////////////////////
